@@ -20,6 +20,7 @@
 #include "qgscomposeritem.h"
 #include <QFile>
 #include <QImage>
+#include <QSvgRenderer>
 
 /** \ingroup MapComposer
  * A composer class that displays svg files or raster format (jpg, png, ...)
@@ -31,6 +32,9 @@ class CORE_EXPORT QgsComposerPicture: public QgsComposerItem
     QgsComposerPicture( QgsComposition *composition );
     ~QgsComposerPicture();
 
+    /** return correct graphics item type. Added in v1.7 */
+    virtual int type() const { return ComposerPicture; }
+
     /**Reimplementation of QCanvasItem::paint*/
     void paint( QPainter* painter, const QStyleOptionGraphicsItem* itemStyle, QWidget* pWidget );
 
@@ -81,18 +85,11 @@ class CORE_EXPORT QgsComposerPicture: public QgsComposerItem
     /**Calculates bounding rect for image such that aspect ratio is correct*/
     QRectF boundedImageRect( double deviceWidth, double deviceHeight );
 
-    /**Updates content of mImage using svg generator*/
-    void updateImageFromSvg();
-
 
     QImage mImage;
+    QSvgRenderer mSVG;
     QFile mSourceFile;
     Mode mMode;
-    /**False if image needs to be rendered from svg*/
-    bool mSvgCacheUpToDate;
-    int mCachedDpi; //store dpis for which the svg cache is valid
-    double mCachedRotation; //store last rotation value to generate new pixmap from svg on change
-    double mCachedViewScaleFactor;
 
     QSize mDefaultSvgSize;
     /**Map that sets the rotation (or 0 if this picture uses map independent rotation)*/
@@ -101,10 +98,6 @@ class CORE_EXPORT QgsComposerPicture: public QgsComposerItem
     double mPictureWidth;
     /**Height of the picture (in mm)*/
     double mPictureHeight;
-
-  signals:
-    /**Tell the configuration widget that the settings need to be updated*/
-    void settingsChanged();
 };
 
 #endif
