@@ -8,17 +8,17 @@
 // <int>.<int>.<int>-<any text>.
 // or else upgrading old project file will not work
 // reliably.
-#define VERSION "1.6.0-Copiapo"
+#define VERSION "1.7.0-Wroclaw"
 
 //used in vim src/core/qgis.cpp
 //The way below should work but it resolves to a number like 0110 which the compiler treats as octal I think
 //because debuggin it out shows the decimal number 72 which results in incorrect version status.
 //As a short term fix I (Tim) am defining the version in top level cmake. It would be good to 
 //reinstate this more generic approach below at some point though
-//#define VERSION_INT 160
-#define VERSION_INT 10600
+//#define VERSION_INT 170
+#define VERSION_INT 10700
 //used in main.cpp and anywhere else where the release name is needed
-#define RELEASE_NAME "Copiapo"
+#define RELEASE_NAME "Wroclaw"
 
 #define QGIS_PLUGIN_SUBDIR "lib/qgis/plugins"
 #define QGIS_DATA_SUBDIR "share/qgis"
