@@ -48,7 +48,7 @@ class QgsMapToPixel;
 class QgsMapLayer;
 class QgsLegend;
 class QgsLegendView;
-class QgsRubberBand;
+class QgsHighlight;
 class QgsVectorLayer;
 
 class QgsMapRenderer;
@@ -77,7 +77,6 @@ class GUI_EXPORT QgsMapCanvasLayer
     const QgsMapLayer* layer() const { return mLayer; }
 
   private:
-
     QgsMapLayer* mLayer;
 
     /** Flag whether layer is visible */
@@ -119,7 +118,11 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     QgsMapRenderer* mapRenderer();
 
     //! Accessor for the canvas pixmap
-    QPixmap& canvasPixmap();
+    //! @deprecated use canvasPaintDevice()
+    Q_DECL_DEPRECATED QPixmap& canvasPixmap();
+
+    //! Accessor for the canvas paint device
+    QPaintDevice &canvasPaintDevice();
 
     //! Get the last reported scale of the canvas
     double scale();
@@ -241,6 +244,9 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     //! used to determine if anti-aliasing is enabled or not
     void enableAntiAliasing( bool theFlag );
 
+    //! true if antialising is enabled
+    bool antiAliasingEnabled() const { return mAntiAliasing; }
+
     //! Select which Qt class to render with
     void useImageToRender( bool theFlag );
 
@@ -368,6 +374,9 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     //! Overridden resize event
     void resizeEvent( QResizeEvent * e );
 
+    //! Overridden paint event
+    void paintEvent( QPaintEvent * e );
+
     //! called when panning is in action, reset indicates end of panning
     void moveCanvasContents( bool reset = false );
 
@@ -451,6 +460,14 @@ class GUI_EXPORT QgsMapCanvas : public QGraphicsView
     //! Mouse wheel action
     WheelAction mWheelAction;
 
+    //! resize canvas size
+    QSize mNewSize;
+
+    //! currently in paint event
+    bool mPainting;
+
+    //! indicates whether antialiasing will be used for rendering
+    bool mAntiAliasing;
 }; // class QgsMapCanvas
 
 
