@@ -12,13 +12,14 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsmaprenderer.h 14373 2010-10-13 19:39:44Z timlinux $ */
+/* $Id$ */
 
 #ifndef QGSMAPRENDER_H
 #define QGSMAPRENDER_H
 
 #include <QSize>
 #include <QStringList>
+#include <QVector>
 
 #include "qgis.h"
 #include "qgsrectangle.h"
@@ -38,12 +39,31 @@ class QgsOverlayObjectPositionManager;
 class QgsVectorLayer;
 class QgsFeature;
 
+struct QgsDiagramLayerSettings;
+
+struct CORE_EXPORT QgsLabelPosition
+{
+  QgsLabelPosition( int id, double r, const QVector< QgsPoint >& corners, const QgsRectangle& rect, double w, double h, const QString& layer, bool upside_down, bool diagram = false ):
+      featureId( id ), rotation( r ), cornerPoints( corners ), labelRect( rect ), width( w ), height( h ), layerID( layer ), upsideDown( upside_down ), isDiagram( diagram ) {}
+  QgsLabelPosition(): featureId( -1 ), rotation( 0 ), labelRect( QgsRectangle() ), width( 0 ), height( 0 ), layerID( "" ), upsideDown( false ), isDiagram( false ) {}
+  int featureId;
+  double rotation;
+  QVector< QgsPoint > cornerPoints;
+  QgsRectangle labelRect;
+  double width;
+  double height;
+  QString layerID;
+  bool upsideDown;
+  bool isDiagram;
+};
+
 /** Labeling engine interface.
  * \note Added in QGIS v1.4
  */
 class QgsLabelingEngineInterface
 {
   public:
+
     virtual ~QgsLabelingEngineInterface() {}
 
     //! called when we're going to start with rendering
@@ -53,12 +73,19 @@ class QgsLabelingEngineInterface
     //! called when starting rendering of a layer
     //! @note: this method was added in version 1.6
     virtual int prepareLayer( QgsVectorLayer* layer, QSet<int>& attrIndices, QgsRenderContext& ctx ) = 0;
+    //! adds a diagram layer to the labeling engine
+    virtual int addDiagramLayer( QgsVectorLayer* layer, QgsDiagramLayerSettings* s ) { return 0; }
     //! called for every feature
     virtual void registerFeature( QgsVectorLayer* layer, QgsFeature& feat, const QgsRenderContext& context = QgsRenderContext() ) = 0;
+    //! called for every diagram feature
+    virtual void registerDiagramFeature( QgsVectorLayer* layer, QgsFeature& feat, const QgsRenderContext& context = QgsRenderContext() ) {};
     //! called when the map is drawn and labels should be placed
     virtual void drawLabeling( QgsRenderContext& context ) = 0;
     //! called when we're done with rendering
     virtual void exit() = 0;
+    //! return infos about labels at a given (map) position
+    //! @note: this method was added in version 1.7
+    virtual QList<QgsLabelPosition> labelsAtPosition( const QgsPoint& p ) = 0;
 
     //! called when passing engine among map renderers
     virtual QgsLabelingEngineInterface* clone() = 0;
@@ -114,7 +141,8 @@ class CORE_EXPORT QgsMapRenderer : public QObject
     void updateScale();
 
     //! Return the measuring object
-    QgsDistanceArea* distanceArea() { return mDistArea; }
+    //! @deprecated
+    Q_DECL_DEPRECATED QgsDistanceArea *distanceArea() { return mDistArea; }
     QGis::UnitType mapUnits() const;
     void setMapUnits( QGis::UnitType u );
 
@@ -148,11 +176,23 @@ class CORE_EXPORT QgsMapRenderer : public QObject
     //! returns true if projections are enabled for this layer set
     bool hasCrsTransformEnabled();
 
-    //! sets destination spatial reference system
-    void setDestinationSrs( const QgsCoordinateReferenceSystem& srs );
+    /** sets destination coordinate reference system
+     * @note deprecated by qgis 1.7
+     * @see setDestinationCrs
+     */
+    Q_DECL_DEPRECATED void setDestinationSrs( const QgsCoordinateReferenceSystem& srs ) { setDestinationCrs( srs ); };
 
-    //! returns CRS of destination spatial reference system
-    const QgsCoordinateReferenceSystem& destinationSrs();
+    /** returns CRS of destination coordinate reference system
+     * @note deprecated by qgis 1.7
+     * @see destinationCrs
+     */
+    Q_DECL_DEPRECATED const QgsCoordinateReferenceSystem& destinationSrs() { return destinationCrs(); };
+
+    //! sets destination coordinate reference system
+    void setDestinationCrs( const QgsCoordinateReferenceSystem& crs );
+
+    //! returns CRS of destination coordinate reference system
+    const QgsCoordinateReferenceSystem& destinationCrs();
 
     void setOutputUnits( OutputUnits u ) {mOutputUnits = u;}
 
@@ -228,7 +268,7 @@ class CORE_EXPORT QgsMapRenderer : public QObject
   protected:
 
     //! indicates drawing in progress
-    bool mDrawing;
+    static bool mDrawing;
 
     //! map units per pixel
     double mMapUnitsPerPixel;
