@@ -1,5 +1,5 @@
 /***************************************************************************
- *   qgsprojectionselector.h
+ *   qgsprojectionselector.h                                               *
  *   Copyright (C) 2005 by Tim Sutton                                      *
  *   tim@linfiniti.com                                                     *
  *                                                                         *
@@ -16,6 +16,8 @@
 #include <QSet>
 #include <QStringList>
 
+#include "qgis.h"
+
 class QResizeEvent;
 
 /** \ingroup gui
@@ -68,7 +70,8 @@ class GUI_EXPORT QgsProjectionSelector: public QWidget, private Ui::QgsProjectio
     const QString sqlSafeString( const QString theSQL );
 
     //! Gets the current EpsgCrsId-style projection identifier
-    long selectedEpsg();
+    // @deprecated there are other authorities - use selectedAuthId()
+    Q_DECL_DEPRECATED long selectedEpsg();
 
     //! Gets the current InternalCrsId-style projection identifier
     long selectedSrsid();
@@ -85,7 +88,9 @@ class GUI_EXPORT QgsProjectionSelector: public QWidget, private Ui::QgsProjectio
 
     void setSelectedAuthId( QString authId );
 
-    void setSelectedEpsg( long epsg );
+    //! Get the selected coordinate system
+    // @deprecated there are other authorities - so not always defined
+    Q_DECL_DEPRECATED void setSelectedEpsg( long epsg );
 
     QString selectedProj4String();
 
@@ -135,7 +140,7 @@ class GUI_EXPORT QgsProjectionSelector: public QWidget, private Ui::QgsProjectio
     QString ogcWmsCrsFilterAsSqlExpression( QSet<QString> * crsFilter );
 
     /**
-     * \brief does the legwork of applying the CRS Name Selection
+     * \brief does the legwork of applying CRS Selection
      *
      * \warning This function does nothing unless getUserList() and getUserProjList()
      *          Have already been called
@@ -144,31 +149,7 @@ class GUI_EXPORT QgsProjectionSelector: public QWidget, private Ui::QgsProjectio
      *          does not scroll the list to the selection if the widget is not visible.
      *          Therefore you will typically want to use this in a showEvent().
      */
-    void applyCRSNameSelection();
-
-    /**
-     * \brief does the legwork of applying the CRS ID Selection
-     *
-     * \warning This function does nothing unless getUserList() and getUserProjList()
-     *          Have already been called
-     *
-     * \warning This function only expands the parents of the selection and
-     *          does not scroll the list to the selection if the widget is not visible.
-     *          Therefore you will typically want to use this in a showEvent().
-     */
-    void applyCRSIDSelection();
-
-    /**
-     * \brief does the legwork of applying the Authority ID Selection
-     *
-     * \warning This function does nothing unless getUserList() and getUserProjList()
-     *          Have already been called
-     *
-     * \warning This function only expands the parents of the selection and
-     *          does not scroll the list to the selection if the widget is not visible.
-     *          Therefore you will typically want to use this in a showEvent().
-     */
-    void applyAuthIDSelection();
+    void applySelection();
 
     /**
        * \brief gets an arbitrary sqlite3 expression from the selection
