@@ -18,7 +18,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsrasterlayer.h 14436 2010-10-26 09:09:51Z jef $ */
+/* $Id$ */
 #ifndef QGSRASTERLAYER_H
 #define QGSRASTERLAYER_H
 
@@ -241,8 +241,8 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
       //added in 1.6 to fix naming glitch
       MultiBandSingleBandGray = MultiBandSingleGandGray, // a layer containing 2 or more bands, but a single band drawn as a range of gray colors
       MultiBandSingleBandPseudoColor, //a layer containing 2 or more bands, but a single band drawn using a pseudocolor algorithm
-      MultiBandColor                  //a layer containing 2 or more bands, mapped to RGB color space.
-      //In the case of a multiband with only two bands, one band will be mapped to more than one color.
+      MultiBandColor,                  //a layer containing 2 or more bands, mapped to RGB color space. In the case of a multiband with only two bands, one band will be mapped to more than one color.
+      SingleBandColorDataStyle         // ARGB values rendered directly
     };
 
     /** \brief This enumerator describes the type of raster layer */
@@ -250,7 +250,8 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     {
       GrayOrUndefined,
       Palette,
-      Multiband
+      Multiband,
+      ColorLayer
     } ;
 
     /** \brief A list containing on ContrastEnhancement object per raster band in this raster layer */
@@ -270,10 +271,6 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
      * band populated, any additional stats are calculated on a need to know basis.*/
     typedef QList<QgsRasterBandStats> RasterStatsList;
 
-
-
-
-
     //
     // Static methods:
     //
@@ -286,22 +283,41 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
      */
     static bool isValidRasterFileName( const QString & theFileNameQString, QString &retError );
     static bool isValidRasterFileName( const QString & theFileNameQString );
-    static QStringList subLayers( GDALDatasetH dataset );
+    //static QStringList subLayers( GDALDatasetH dataset );
 
 
     /** Return time stamp for given file name */
     static QDateTime lastModified( const QString &  name );
 
+    // Keep this for now, it is used by Python interface!!!
     /** \brief ensures that GDAL drivers are registered, but only once */
     static void registerGdalDrivers();
 
-
-
-
     //
     // Non Static inline methods
     //
 
+    /** \brief Initialize default values */
+    void init();
+
+    // For backward compatibility (Python) get rid of it once python is updated
+    void setDataProvider( const QString & provider,
+                          const QStringList & layers,
+                          const QStringList & styles,
+                          const QString & format,
+                          const QString & crs );
+    /**  [ data provider interface ] Set the data provider */
+    void setDataProvider( const QString & provider,
+                          const QStringList & layers,
+                          const QStringList & styles,
+                          const QString & format,
+                          const QString & crs,
+                          bool loadDefaultStyleFlag );
+
+    static QLibrary* loadProviderLibrary( QString theProviderKey );
+    static QgsRasterDataProvider* loadProvider( QString theProviderKey, QString theDataSource = 0 );
+
+
     /** \brief  Accessor for blue band name mapping */
     QString blueBandName() const { return mBlueBandName; }
 
@@ -362,12 +378,6 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     /** \brief Accessor for red band name (allows alternate mappings e.g. map blue as red color) */
     QString redBandName() const { return mRedBandName; }
 
-    /**  [ data provider interface ] Set the data provider */
-    void setDataProvider( const QString & provider,
-                          const QStringList & layers,
-                          const QStringList & styles,
-                          const QString & format,
-                          const QString & crs );
 
     /** \brief Mutator for drawing style */
     void setDrawingStyle( const DrawingStyle &  theDrawingStyle ) { mDrawingStyle = theDrawingStyle; }
@@ -402,10 +412,6 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     /** \brief Accessor that returns the width of the (unclipped) raster  */
     int width() { return mWidth; }
 
-
-
-
-
     //
     // Non Static methods
     //
@@ -544,12 +550,13 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     QPixmap paletteAsPixmap( int theBand = 1 );
 
     /**  \brief [ data provider interface ] Which provider is being used for this Raster Layer? */
-    QString providerKey();
+    QString providerKey() const;
 
     /** \brief Returns the number of raster units per each raster pixel. In a world file, this is normally the first row (without the sign) */
     double rasterUnitsPerPixel();
 
     /** \brief Read color table from GDAL raster band */
+    // Keep this for QgsRasterLayerProperties
     bool readColorTable( int theBandNumber, QList<QgsColorRampShader::ColorRampItem>* theList );
 
     /** \brief Simple reset function that set the noDataValue back to the value stored in the first raster band */
@@ -589,9 +596,15 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     /** \brief Mutator for setting the maximum value for contrast enhancement */
     void setMaximumValue( QString theBand, double theValue, bool theGenerateLookupTableFlag = true );
 
-    /** \brief Sets the minimum and maximum values for the band(s) currently being displayed using the only pixel values from the last/current extent */
+    /** \brief Sets the minimum and maximum values for the band(s) currently
+     * being displayed using the only pixel values from the last/current extent
+     * */
     void setMinimumMaximumUsingLastExtent();
 
+    /** \brief Sets the minimum and maximum values for the band(s) currently
+     * being displayed using the only pixel values from the dataset min/max */
+    void setMinimumMaximumUsingDataset();
+
     /** \brief Mutator for setting the minimum value for contrast enhancement */
     void setMinimumValue( unsigned int theBand, double theValue, bool theGenerateLookupTableFlag = true );
 
@@ -627,10 +640,6 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     /** \brief Emit a signal asking for a repaint. (inherited from maplayer) */
     void triggerRepaint();
 
-
-
-
-
     //
     // Virtural methods
     //
@@ -647,9 +656,8 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
      */
     virtual void setSubLayerVisibility( const QString & name, bool vis );
 
-
-
-
+    /** Time stamp of data source in the moment when data/metadata were loaded by provider */
+    virtual QDateTime timestamp() const { return mDataProvider->timestamp() ; }
 
   public slots:
     /** \brief Create GDAL pyramid overviews */
@@ -658,6 +666,7 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
                            bool theTryInternalFlag = false );
 
     /** \brief Populate the histogram vector for a given band */
+
     void populateHistogram( int theBandNoInt,
                             int theBinCountInt = 256,
                             bool theIgnoreOutOfRangeFlag = true,
@@ -668,16 +677,18 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     /** \brief Propagate progress updates from GDAL up to the parent app */
     void updateProgress( int, int );
 
-
-
-
+    /** \brief recieve progress signal from provider */
+    void onProgress( int, double, QString );
 
   signals:
     /** \brief Signal for notifying listeners of long running processes */
     void progressUpdate( int theValue );
 
-
-
+    /**
+     *   This is emitted whenever data or metadata (e.g. color table, extent) has changed
+     *   @note added in 1.7
+     */
+    void dataChanged();
 
   protected:
 
@@ -693,14 +704,15 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     /** \brief Write layer specific state to project file Dom node */
     bool writeXml( QDomNode & layer_node, QDomDocument & doc );
 
-
-
-
-
   private:
     //
     // Private methods
     //
+    /** \brief Drawing routine for color type data  */
+    void drawSingleBandColorData( QPainter * theQPainter,
+                                  QgsRasterViewPort * theRasterViewPort,
+                                  const QgsMapToPixel* theQgsMapToPixel,
+                                  int theBandNoInt );
 
     /** \brief Drawing routine for multiband image  */
     void drawMultiBandColor( QPainter * theQPainter,
@@ -755,8 +767,8 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
                                     const QgsMapToPixel* theQgsMapToPixel,
                                     int theBandNoInt );
 
-    /** \brief Close data set and release related data */
-    void closeDataset();
+    /** \brief Close data provider and clear related members */
+    void closeDataProvider();
 
     /** \brief helper function to create zero padded band names */
     QString  generateBandName( int );
@@ -764,21 +776,19 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     /** \brief Find out whether a given band exists.    */
     bool hasBand( const QString &  theBandName );
 
-    /** \brief Places the rendered image onto the canvas */
-    void paintImageToCanvas( QPainter* theQPainter, QgsRasterViewPort * theRasterViewPort,
-                             const QgsMapToPixel* theQgsMapToPixel, QImage* theImage );
-
     /** \brief Query GDAL to find out the Wkt projection string for this layer.*/
     QString projectionWkt();
 
     /** \brief Allocate memory and load data to that allocated memory */
-    void* readData( GDALRasterBandH gdalBand, QgsRasterViewPort *viewPort );
+    //void* readData( GDALRasterBandH gdalBand, QgsRasterViewPort *viewPort );
+    void* readData( int bandNo, QgsRasterViewPort *viewPort );
 
     /** \brief Load the given raster file */
     bool readFile( const QString & fileName );
 
     /** \brief Read a raster value given position from memory block created by readData() */
-    inline double readValue( void *data, GDALDataType type, int index );
+    //inline double readValue( void *data, GDALDataType type, int index );
+    inline double readValue( void *data, int type, int index );
 
     /** \brief Update the layer if it is outdated */
     bool update();
@@ -786,9 +796,6 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     /** \brief Verify and transform band name for internal consistency. Return 'Not Set' on any type of failure */
     QString validateBandName( const QString & theBandName );
 
-
-
-
     //
     // Private member vars
     //
@@ -829,10 +836,10 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     QString mErrorCaption;
 
     /** \brief Pointer to the gdaldataset */
-    GDALDatasetH mGdalBaseDataset;
+    //GDALDatasetH mGdalBaseDataset;
 
     /** \brief Pointer to the gdaldataset (possibly warped vrt) */
-    GDALDatasetH mGdalDataset;
+    //GDALDatasetH mGdalDataset;
 
     /** \brief Values for mapping pixel to world coordinates. Contents of this array are the same as the GDAL adfGeoTransform */
     double mGeoTransform[6];
@@ -864,7 +871,7 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
     QgsRasterViewPort mLastViewPort;
 
     /**  [ data provider interface ] pointer for loading the provider library */
-    QLibrary* mLib;
+    //QLibrary* mLib;
 
     /**  [ data provider interface ] Flag indicating whether the layer has been modified since the last commit*/
     bool mModified;
@@ -906,6 +913,13 @@ class CORE_EXPORT QgsRasterLayer : public QgsMapLayer
 
     /** \brief Flag indicating if the nodatavalue is valid*/
     bool mValidNoDataValue;
+
+    /** WMS parameters */
+    /* TODO: put everything to URI */
+    QStringList mLayers;
+    QStringList mStyles;
+    QString mFormat;
+    QString mCrs;
 };
 
 /*#include <QColor>
@@ -922,7 +936,7 @@ class QPainter;*/
 class CORE_EXPORT QgsRasterImageBuffer
 {
   public:
-    QgsRasterImageBuffer( GDALRasterBandH rasterBand, QPainter* p,
+    QgsRasterImageBuffer( QgsRasterDataProvider *dataProvider, int bandNo, QPainter* p,
                           QgsRasterViewPort* viewPort, const QgsMapToPixel* mapToPixel, double* mGeoTransform );
     ~QgsRasterImageBuffer();
     void reset( int maxPixelsInVirtualMemory = 5000000 );
@@ -939,7 +953,8 @@ class CORE_EXPORT QgsRasterImageBuffer
     /**Peter's fix for zoomed in rasters*/
     void drawPixelRectangle();
 
-    GDALRasterBandH mRasterBand; //raster band
+    QgsRasterDataProvider* mDataProvider;
+    int mBandNo;
     QPainter* mPainter;
     QgsRasterViewPort* mViewPort;
     const QgsMapToPixel* mMapToPixel;
@@ -958,6 +973,8 @@ class CORE_EXPORT QgsRasterImageBuffer
     int mCurrentPartImageRow; //current image row
     int mNumCurrentImageRows; //number of image rows for the current part
 
+    int mCurrentPart;
+
     //current memory image and gdal scan data
     QImage* mCurrentImage;
     void* mCurrentGDALData;
