@@ -15,7 +15,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgssearchtreenode.h 14394 2010-10-18 00:13:51Z jef $ */
+/* $Id$ */
 
 #ifndef QGSSEARCHTREENODE_H
 #define QGSSEARCHTREENODE_H
@@ -26,6 +26,7 @@
 #include <QVariant>
 #include <QList>
 
+#include <qgis.h>
 #include <qgsfield.h>
 #include <qgsfeature.h>
 
@@ -74,15 +75,21 @@ class CORE_EXPORT QgsSearchTreeNode
       opASIN,
       opACOS,
       opATAN,
+      opATAN2,
 
       // conversion
       opTOINT,
       opTOREAL,
       opTOSTRING,
 
+      // coordinates
+      opX,
+      opY,
+
       // measuring
       opLENGTH,
       opAREA,
+      opPERIMETER,
 
       // feature id
       opID,
@@ -106,6 +113,9 @@ class CORE_EXPORT QgsSearchTreeNode
       opCONCAT,
       opLOWER,
       opUPPER,
+      opREPLACE,
+      opSTRLEN,
+      opSUBSTR,
 
       opROWNUM
     };
@@ -150,8 +160,8 @@ class CORE_EXPORT QgsSearchTreeNode
     //! @note attribute and optional geom parameter replaced with feature in 1.6
     bool checkAgainst( const QgsFieldMap& fields, QgsFeature &f );
 
-    //! @note deprecated
-    bool checkAgainst( const QgsFieldMap& fields, const QgsAttributeMap& attributes, QgsGeometry* geom = 0 );
+    //! @deprecated
+    Q_DECL_DEPRECATED bool checkAgainst( const QgsFieldMap& fields, const QgsAttributeMap& attributes, QgsGeometry* geom = 0 );
 
     //! checks if there were errors during evaluation
     bool hasError() { return ( !mError.isEmpty() ); }
@@ -166,12 +176,12 @@ class CORE_EXPORT QgsSearchTreeNode
                    const QgsFieldMap& fields,
                    QgsFeature &f );
 
-    //! @note deprecated
-    bool getValue( QgsSearchTreeValue& value,
-                   QgsSearchTreeNode* node,
-                   const QgsFieldMap &fields,
-                   const QgsAttributeMap &attributes,
-                   QgsGeometry* geom = 0 );
+    //! @deprecated
+    Q_DECL_DEPRECATED bool getValue( QgsSearchTreeValue& value,
+                                     QgsSearchTreeNode* node,
+                                     const QgsFieldMap &fields,
+                                     const QgsAttributeMap &attributes,
+                                     QgsGeometry* geom = 0 );
 
     //! return a list of referenced columns in the tree
     //! @note added in 1.5
@@ -206,8 +216,8 @@ class CORE_EXPORT QgsSearchTreeNode
     //! returns scalar value of node
     QgsSearchTreeValue valueAgainst( const QgsFieldMap& fields, QgsFeature &f );
 
-    //! @note deprecated
-    QgsSearchTreeValue valueAgainst( const QgsFieldMap& fields, const QgsAttributeMap& attributes, QgsGeometry* geom = 0 );
+    //! @deprecated
+    Q_DECL_DEPRECATED QgsSearchTreeValue valueAgainst( const QgsFieldMap& fields, const QgsAttributeMap& attributes, QgsGeometry* geom = 0 );
 
     //! strips mText when node is of string type
     void stripText();
