@@ -12,7 +12,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgsvectordataprovider.h 14319 2010-09-30 22:53:18Z jef $ */
+/* $Id$ */
 #ifndef QGSVECTORDATAPROVIDER_H
 #define QGSVECTORDATAPROVIDER_H
 
@@ -25,8 +25,13 @@ class QTextCodec;
 //QGIS Includes
 #include "qgis.h"
 #include "qgsdataprovider.h"
-#include "qgsvectorlayer.h"
+#include "qgsfeature.h"
 #include "qgsfield.h"
+#include "qgsrectangle.h"
+
+typedef QList<int> QgsAttributeList;
+typedef QSet<int> QgsFeatureIds;
+typedef QSet<int> QgsAttributeIds;
 
 /** \ingroup core
  * This is the base class for vector data providers.
@@ -74,6 +79,7 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider
       RandomSelectGeometryAtId =     1 << 10,
       /** DEPRECATED - do not use */
       SequentialSelectGeometryAtId = 1 << 11,
+      CreateAttributeIndex = 1 << 12
     };
 
     /** bitmask of all provider's editing capabilities */
@@ -234,9 +240,9 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider
      * Add new attributes
      * @param attributes map of attributes name as key and type as value
      * @return true in case of success and false in case of failure
-     * @note deprecated
+     * @deprecated
      */
-    virtual bool addAttributes( const QMap<QString, QString> &attributes );
+    Q_DECL_DEPRECATED virtual bool addAttributes( const QMap<QString, QString> &attributes );
 
     /**
      * Deletes existing attributes
@@ -272,6 +278,9 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider
      */
     virtual bool createSpatialIndex();
 
+    /**Create an attribute index on the datasource*/
+    virtual bool createAttributeIndex( int field );
+
     /** Returns a bitmask containing the supported capabilities
         Note, some capabilities may change depending on whether
         a spatial filter is active on this provider, so it may
@@ -341,9 +350,9 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider
 
     /**
      * Returns the names of the supported types
-     * @note deprecated
+     * @deprecated use nativeTypes()
      */
-    const QMap<QString, QVariant::Type> &supportedNativeTypes() const;
+    Q_DECL_DEPRECATED const QMap<QString, QVariant::Type> &supportedNativeTypes() const;
 
     /** Returns true if the provider is strict about the type of inserted features
           (e.g. no multipolygon in a polygon layer)
@@ -353,6 +362,22 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider
 
     static const QStringList &availableEncodings();
 
+    /* provider has errors to report
+     * @note added in 1.7
+     */
+    bool hasErrors();
+
+    /* clear recorded errors
+     * @note added in 1.7
+     */
+    void clearErrors();
+
+    /* get recorded errors
+     * @note added in 1.7
+     */
+    QStringList errors();
+
+
   protected:
     QVariant convertValue( QVariant::Type type, QString value );
 
@@ -377,11 +402,18 @@ class CORE_EXPORT QgsVectorDataProvider : public QgsDataProvider
     /**The names of the providers native types*/
     QList< NativeType > mNativeTypes;
 
+    void pushError( QString msg );
+
   private:
     /** old notation **/
     QMap<QString, QVariant::Type> mOldTypeList;
 
+    // list of errors
+    QStringList mErrors;
+
+
     static QStringList smEncodings;
+
 };
 
 #endif
