@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """
 /***************************************************************************
-Name			 	 : GdalTools
+Name                  : GdalTools
 Description          : Integrate gdal tools into qgis
 Date                 : 17/Sep/09
 copyright            : (C) 2009 by Lorenzo Masini (Faunalia)
@@ -32,7 +32,7 @@ try:
   from osgeo import gdal
   from osgeo import ogr
 except ImportError, e:
-  error_str = str(e)
+  error_str = e.args[0]
   error_mod = error_str.replace( "No module named ", "" )
   if req_mods.has_key( error_mod ):
     error_str = error_str.replace( error_mod, req_mods[error_mod] )
@@ -78,14 +78,34 @@ class GdalTools:
       + QCoreApplication.translate( "GdalTools", "This version of Gdal Tools requires at least QGIS version 1.0.0\nPlugin will not be enabled." ) )
       return None
 
-    from tools.GdalTools_utils import Version, GdalConfig
+    from tools.GdalTools_utils import Version, GdalConfig, LayerRegistry
     self.GdalVersion = Version( GdalConfig.version() )
+    LayerRegistry.setIface( self.iface )
 
-    self.menu = QMenu()
-    self.menu.setTitle( QCoreApplication.translate( "GdalTools", "&Raster" ) )
+    # find the Raster menu
+    rasterMenu = None
+    menu_bar = self.iface.mainWindow().menuBar()
+    actions = menu_bar.actions()
+
+    rasterText = QCoreApplication.translate( "QgisApp", "&Raster" )
+
+    for a in actions:
+        if a.menu() != None and a.menu().title() == rasterText:
+            rasterMenu = a.menu()
+            break
+
+    if rasterMenu == None:
+        # no Raster menu, create and insert it before the Help menu
+        self.menu = QMenu()
+        self.menu.setTitle( rasterText )
+        lastAction = actions[ len( actions ) - 1 ]
+        menu_bar.insertMenu( lastAction, self.menu )
+    else:
+        self.menu = rasterMenu
+        self.menu.addSeparator()
 
     if self.GdalVersion >= "1.6":
-      self.buildVRT = QAction( QIcon(":/icons/vrt.png"), QCoreApplication.translate( "GdalTools", "Build Virtual Raster (catalog)" ), self.iface.mainWindow() )
+      self.buildVRT = QAction( QIcon(":/icons/vrt.png"), QCoreApplication.translate( "GdalTools", "Build Virtual Raster (Catalog)" ), self.iface.mainWindow() )
       self.buildVRT.setStatusTip( QCoreApplication.translate( "GdalTools", "Builds a VRT from a list of datasets") )
       QObject.connect( self.buildVRT, SIGNAL( "triggered()" ), self.doBuildVRT )
       self.menu.addAction(self.buildVRT)
@@ -97,13 +117,13 @@ class GdalTools:
       self.menu.addAction(self.contour)
 
     if self.GdalVersion >= "1.3":
-      self.rasterize = QAction( QIcon(":/icons/rasterize.png"), QCoreApplication.translate( "GdalTools", "Rasterize" ), self.iface.mainWindow() )
+      self.rasterize = QAction( QIcon(":/icons/rasterize.png"), QCoreApplication.translate( "GdalTools", "Rasterize (Vector to raster)" ), self.iface.mainWindow() )
       self.rasterize.setStatusTip( QCoreApplication.translate( "GdalTools", "Burns vector geometries into a raster") )
       QObject.connect( self.rasterize, SIGNAL( "triggered()" ), self.doRasterize )
       self.menu.addAction(self.rasterize)
 
     if self.GdalVersion >= "1.6":
-      self.polygonize = QAction( QIcon(":/icons/polygonize.png"), QCoreApplication.translate( "GdalTools", "Polygonize" ), self.iface.mainWindow() )
+      self.polygonize = QAction( QIcon(":/icons/polygonize.png"), QCoreApplication.translate( "GdalTools", "Polygonize (Raster to vector)" ), self.iface.mainWindow() )
       self.polygonize.setStatusTip( QCoreApplication.translate( "GdalTools", "Produces a polygon feature layer from a raster") )
       QObject.connect( self.polygonize, SIGNAL( "triggered()" ), self.doPolygonize )
       self.menu.addAction(self.polygonize)
@@ -120,7 +140,7 @@ class GdalTools:
       self.menu.addAction(self.sieve)
 
     if self.GdalVersion >= "1.6":
-      self.proximity = QAction( QIcon(":/icons/proximity.png"),  QCoreApplication.translate( "GdalTools", "Proximity" ), self.iface.mainWindow() )
+      self.proximity = QAction( QIcon(":/icons/proximity.png"),  QCoreApplication.translate( "GdalTools", "Proximity (Raster distance)" ), self.iface.mainWindow() )
       self.proximity.setStatusTip( QCoreApplication.translate( "GdalTools", "Produces a raster proximity map") )
       QObject.connect( self.proximity, SIGNAL( "triggered()" ), self.doProximity )
       self.menu.addAction(self.proximity)
@@ -131,18 +151,18 @@ class GdalTools:
       QObject.connect( self.nearBlack, SIGNAL( "triggered()" ), self.doNearBlack )
       self.menu.addAction(self.nearBlack)
 
-    self.warp = QAction( QIcon(":/icons/warp.png"),  QCoreApplication.translate( "GdalTools", "Warp" ), self.iface.mainWindow() )
+    self.warp = QAction( QIcon(":/icons/warp.png"),  QCoreApplication.translate( "GdalTools", "Warp (Reproject)" ), self.iface.mainWindow() )
     self.warp.setStatusTip( QCoreApplication.translate( "GdalTools", "Warp an image into a new coordinate system") )
     QObject.connect( self.warp, SIGNAL( "triggered()" ), self.doWarp )
     self.menu.addAction(self.warp)
 
     if self.GdalVersion >= "1.5":
-      self.grid = QAction( QIcon(":/icons/grid.png"), QCoreApplication.translate( "GdalTools", "Grid" ), self.iface.mainWindow() )
+      self.grid = QAction( QIcon(":/icons/grid.png"), QCoreApplication.translate( "GdalTools", "Grid (Interpolation)" ), self.iface.mainWindow() )
       self.grid.setStatusTip( QCoreApplication.translate( "GdalTools", "Create raster from the scattered data") )
       QObject.connect( self.grid, SIGNAL( "triggered()" ), self.doGrid )
       self.menu.addAction(self.grid)
 
-    self.translate = QAction( QIcon(":/icons/translate.png"), QCoreApplication.translate( "GdalTools", "Translate" ), self.iface.mainWindow() )
+    self.translate = QAction( QIcon(":/icons/translate.png"), QCoreApplication.translate( "GdalTools", "Translate (Convert format)" ), self.iface.mainWindow() )
     self.translate.setStatusTip( QCoreApplication.translate( "GdalTools", "Converts raster data between different formats") )
     QObject.connect( self.translate, SIGNAL( "triggered()" ), self.doTranslate )
     self.menu.addAction(self.translate)
@@ -157,7 +177,7 @@ class GdalTools:
     QObject.connect( self.projection, SIGNAL( "triggered()" ), self.doProjection )
     self.menu.addAction( self.projection )
 
-    self.overview = QAction( QIcon( ":icons/raster-overview.png" ), QCoreApplication.translate( "GdalTools", "Build overviews" ), self.iface.mainWindow() )
+    self.overview = QAction( QIcon( ":icons/raster-overview.png" ), QCoreApplication.translate( "GdalTools", "Build overviews (Pyramids)" ), self.iface.mainWindow() )
     self.overview.setStatusTip( QCoreApplication.translate( "GdalTools", "Builds or rebuilds overview images" ) )
     QObject.connect( self.overview, SIGNAL( "triggered()" ), self.doOverview )
     self.menu.addAction( self.overview )
@@ -177,6 +197,17 @@ class GdalTools:
     QObject.connect( self.rgb, SIGNAL( "triggered()" ), self.doRGB )
     self.menu.addAction(self.rgb)
 
+    self.tileindex = QAction( QIcon( ":icons/tileindex.png" ), QCoreApplication.translate( "GdalTools", "Tile index" ), self.iface.mainWindow() )
+    self.tileindex.setStatusTip( QCoreApplication.translate( "GdalTools", "Build a shapefile as a raster tileindex" ) )
+    QObject.connect( self.tileindex, SIGNAL( "triggered()" ), self.doTileIndex )
+    self.menu.addAction(self.tileindex)
+
+    if self.GdalVersion >= "1.7":
+      self.dem = QAction( QIcon( ":icons/dem.png" ), QCoreApplication.translate( "GdalTools", "DEM (Terrain models)" ), self.iface.mainWindow() )
+      self.dem.setStatusTip( QCoreApplication.translate( "GdalTools", "Tool to analyze and visualize DEMs" ) )
+      QObject.connect( self.dem, SIGNAL( "triggered()" ), self.doDEM )
+      self.menu.addAction(self.dem)
+
     self.settings = QAction( QCoreApplication.translate( "GdalTools", "GdalTools settings" ), self.iface.mainWindow() )
     self.settings.setStatusTip( QCoreApplication.translate( "GdalTools", "Various settings for Gdal Tools" ) )
     QObject.connect( self.settings, SIGNAL( "triggered()" ), self.doSettings )
@@ -199,87 +230,102 @@ class GdalTools:
   def doBuildVRT( self ):
     from tools.doBuildVRT import GdalToolsDialog as BuildVRT
     d = BuildVRT( self.iface )
-    d.show_()
+    self.runToolDialog( d )
 
   def doContour( self ):
     from tools.doContour import GdalToolsDialog as Contour
     d = Contour( self.iface )
-    d.show_()
+    self.runToolDialog( d )
 
   def doRasterize( self ):
     from tools.doRasterize import GdalToolsDialog as Rasterize
     d = Rasterize( self.iface )
-    d.show_()
+    self.runToolDialog( d )
 
   def doPolygonize( self ):
     from tools.doPolygonize import GdalToolsDialog as Polygonize
     d = Polygonize( self.iface )
-    d.show_()
+    self.runToolDialog( d )
 
   def doMerge( self ):
     from tools.doMerge import GdalToolsDialog as Merge
     d = Merge( self.iface )
-    d.show_()
+    self.runToolDialog( d )
 
   def doSieve( self ):
     from tools.doSieve import GdalToolsDialog as Sieve
     d = Sieve( self.iface )
-    d.show_()
+    self.runToolDialog( d )
 
   def doProximity( self ):
     from tools.doProximity import GdalToolsDialog as Proximity
     d = Proximity( self.iface )
-    d.show_()
+    self.runToolDialog( d )
 
   def doNearBlack( self ):
     from tools.doNearBlack import GdalToolsDialog as NearBlack
     d = NearBlack( self.iface )
-    d.show_()
+    self.runToolDialog( d )
 
   def doWarp( self ):
     from tools.doWarp import GdalToolsDialog as Warp
     d = Warp( self.iface )
-    d.show_()
+    self.runToolDialog( d )
 
   def doGrid( self ):
     from tools.doGrid import GdalToolsDialog as Grid
     d = Grid( self.iface )
-    d.show_()
+    self.runToolDialog( d )
 
   def doTranslate( self ):
     from tools.doTranslate import GdalToolsDialog as Translate
     d = Translate( self.iface )
-    d.show_()
+    self.runToolDialog( d )
 
   def doInfo( self ):
     from tools.doInfo import GdalToolsDialog as Info
     d = Info( self.iface )
-    d.show_()
+    self.runToolDialog( d )
 
   def doProjection( self ):
     from tools.doProjection import GdalToolsDialog as Projection
     d = Projection( self.iface )
-    d.show_()
+    self.runToolDialog( d )
 
   def doOverview( self ):
     from tools.doOverview import GdalToolsDialog as Overview
     d = Overview( self.iface )
-    d.show_()
+    self.runToolDialog( d )
 
   def doClipper( self ):
     from tools.doClipper import GdalToolsDialog as Clipper
     d = Clipper( self.iface )
-    d.show_()
+    self.runToolDialog( d )
 
   def doPaletted( self ):
     from tools.doRgbPct import GdalToolsDialog as RgbPct
     d = RgbPct( self.iface )
-    d.show_()
+    self.runToolDialog( d )
 
   def doRGB( self ):
     from tools.doPctRgb import GdalToolsDialog as PctRgb
     d = PctRgb( self.iface )
-    d.show_()
+    self.runToolDialog( d )
+
+  def doTileIndex( self ):
+    from tools.doTileIndex import GdalToolsDialog as TileIndex
+    d = TileIndex( self.iface )
+    self.runToolDialog( d )
+
+  def doDEM( self ):
+    from tools.doDEM import GdalToolsDialog as DEM
+    d = DEM( self.iface )
+    self.runToolDialog( d )
+
+  def runToolDialog( self, dlg ):
+    dlg.show_()
+    dlg.exec_()
+    del dlg
 
   def doSettings( self ):
     from tools.doSettings import GdalToolsSettingsDialog as Settings
