@@ -8,8 +8,10 @@
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
+
 from qgis.core import *
 from qgis.gui import *
+
 from osgeo import gdal
 from osgeo.gdalconst import *
 from osgeo import ogr
@@ -54,6 +56,16 @@ def setGdalPath( path ):
   settings = QSettings()
   settings.setValue( "/GdalTools/gdalPath", QVariant( path ) )
 
+# Retrieves GDAL help files location
+def getHelpPath():
+  settings = QSettings()
+  return settings.value( "/GdalTools/helpPath", QVariant( "" ) ).toString()
+
+# Stores GDAL help files location
+def setHelpPath( path ):
+  settings = QSettings()
+  settings.setValue( "/GdalTools/helpPath", QVariant( path ) )
+
 # Retrieves last used encoding from persistent settings
 def getLastUsedEncoding():
     settings = QSettings()
@@ -80,32 +92,97 @@ def getVectorExtensions():
     extensions << FileFilter.getFilterExtensions( f )
   return extensions
 
-def getRasterLayers():
-  # mantein the reference between combobox indexes and canvas layers
-  layers = dict()
-  count = 0
-  names = []
-  layerMap = QgsMapLayerRegistry.instance().mapLayers()
-  for name, layer in layerMap.iteritems():
-    # only raster layers, but not WMS ones
-    if layer.type() == layer.RasterLayer and ( not layer.usesProvider() ):
-      layers[count] = layer
-      names.append(layer.name())
-      count = count +1
-  return (layers, names)
+class LayerRegistry(QObject):
 
-def getVectorLayers():
-  # mantein the reference between combobox indexes and canvas layers
-  layers = dict()
-  count = 0
-  names = []
-  layerMap = QgsMapLayerRegistry.instance().mapLayers()
-  for name, layer in layerMap.iteritems():
-    if layer.type() == layer.VectorLayer:
-      layers[count] = layer
-      names.append(layer.name())
-      count = count +1
-  return (layers, names)
+    _instance = None
+    _iface = None
+
+    @staticmethod
+    def instance():
+      if LayerRegistry._instance == None:
+        LayerRegistry._instance = LayerRegistry()
+      return LayerRegistry._instance
+
+    @staticmethod
+    def setIface(iface):
+      LayerRegistry._iface = iface
+
+    layers = []
+
+    def __init__(self):
+      QObject.__init__(self)
+      if LayerRegistry._instance != None:
+        return
+
+      LayerRegistry.layers = self.getAllLayers()
+      LayerRegistry._instance = self
+      self.connect(QgsMapLayerRegistry.instance(), SIGNAL("removedAll()"), self.removeAllLayers)
+      self.connect(QgsMapLayerRegistry.instance(), SIGNAL("layerWasAdded(QgsMapLayer *)"), self.layerAdded)
+      self.connect(QgsMapLayerRegistry.instance(), SIGNAL("layerWillBeRemoved(QString)"), self.removeLayer)
+
+    def getAllLayers(self):
+       if LayerRegistry._iface and hasattr(LayerRegistry._iface, 'legendInterface'):
+         return LayerRegistry._iface.legendInterface().layers()
+       return QgsMapLayerRegistry.instance().mapLayers().values()
+
+    def layerAdded(self, layer):
+       LayerRegistry.layers.append( layer )
+       self.emit( SIGNAL( "layersChanged" ) )
+
+    def removeLayer(self, layerId):
+       LayerRegistry.layers = filter( lambda x: x.getLayerID() != layerId, LayerRegistry.layers)
+       self.emit( SIGNAL( "layersChanged" ) )
+
+    def removeAllLayers(self):
+       LayerRegistry.layers = []
+       self.emit( SIGNAL( "layersChanged" ) )
+
+    @classmethod
+    def isRaster(self, layer):
+      # only gdal raster layers
+      if layer.type() != layer.RasterLayer:
+        return False
+      if layer.usesProvider() and layer.providerKey() != 'gdal':
+        return False
+      return True
+
+    def getRasterLayers(self):
+      return filter( self.isRaster, LayerRegistry.layers )
+
+    @classmethod
+    def isVector(self, layer):
+      if layer.type() != layer.VectorLayer:
+        return False
+      return True
+
+    def getVectorLayers(self):
+      return filter( self.isVector, LayerRegistry.layers )
+
+def getRasterFiles(path, recursive=False):
+  rasters = QStringList()
+  if not QFileInfo(path).exists():
+    return rasters
+
+  filter = getRasterExtensions()
+  workDir = QDir( path )
+  workDir.setFilter( QDir.Files | QDir.NoSymLinks | QDir.NoDotAndDotDot )
+  workDir.setNameFilters( filter )
+  files = workDir.entryList()
+  for f in files:
+    rasters << path + "/" + f
+
+  if recursive:
+    import os
+    for myRoot, myDirs, myFiles in os.walk( unicode(path) ):
+      for dir in myDirs:
+        workDir = QDir( myRoot + "/" + dir )
+        workDir.setFilter( QDir.Files | QDir.NoSymLinks | QDir.NoDotAndDotDot )
+        workDir.setNameFilters( filter )
+        workFiles = workDir.entryList()
+        for f in workFiles:
+          rasters << myRoot + "/" + dir + "/" + f
+
+  return rasters
 
 def fillRasterOutputFormat(aFilter = None, filename = None):
   shortName = QString()
@@ -145,10 +222,15 @@ def fillVectorOutputFormat(aFilter = None, filename = None):
 
   return shortName
 
+class UnsupportedOGRFormat(Exception):
+    def __init__(self): 
+      msg = QCoreApplication.translate( "GdalTools", "The selected file is not a supported OGR format" )
+      Exception.__init__(self, msg) 
+
 def getVectorFields(vectorFile):
     hds = ogr.Open( unicode(vectorFile).encode('utf8') )
     if hds == None:
-      raise Exception( QCoreApplication.translate( "GdalTools", "The selected file is not a supported OGR format" ) )
+      raise UnsupportedOGRFormat()
 
     fields = []
     names = []
@@ -174,10 +256,12 @@ def getRasterSRS( parent, fileName ):
       arr = processSRS.readAllStandardOutput()
       processSRS.close()
 
-    if not arr.isEmpty():
-      info = QString( arr ).split( "\n" ).filter( "AUTHORITY" )
-      if info.count() == 0:
-        return QString()
+    if arr.isEmpty():
+      return QString()
+
+    info = QString( arr ).split( "\n" ).filter( "AUTHORITY" )
+    if info.count() == 0:
+      return QString()
 
     srs = info[ info.count() - 1 ]
     srs = srs.simplified().remove( "AUTHORITY[" )
@@ -186,6 +270,30 @@ def getRasterSRS( parent, fileName ):
     srs = info[ 0 ] + ":" + info[ 1 ]
     return srs
 
+def getRasterExtent(parent, fileName):
+    processSRS = QProcess( parent )
+    processSRS.start( "gdalinfo", QStringList() << fileName, QIODevice.ReadOnly )
+    arr = QByteArray()
+    if processSRS.waitForFinished():
+      arr = processSRS.readAllStandardOutput()
+      processSRS.close()
+      
+    if arr.isEmpty():
+      return
+      
+    info = QString( arr ).split( "\n" )
+    ulCoord = info[ info.indexOf( QRegExp( "^Upper\sLeft.*" ) ) ].simplified()
+    lrCoord = info[ info.indexOf( QRegExp( "^Lower\sRight.*" ) ) ].simplified()
+    ulCoord = ulCoord[ulCoord.indexOf( "(" ) + 1 : ulCoord.indexOf( ")" ) - 1].split( "," )
+    lrCoord = lrCoord[lrCoord.indexOf( "(" ) + 1 : lrCoord.indexOf( ")" ) - 1].split( "," )
+    xUL = ulCoord[0].toDouble()[0]
+    yUL = ulCoord[1].toDouble()[0]
+    xLR = lrCoord[0].toDouble()[0]
+    yLR = lrCoord[1].toDouble()[0]
+
+    return QgsRectangle( xUL, yLR, xLR, yUL )
+
+
 # This class is used to replace the QFileDialog class.
 # Its static methods are used in place of the respective QFileDialog ones to:
 # 1. set the last used directory
@@ -284,7 +392,7 @@ class FileFilter:
 
       # workaround for QGis < 1.5 (see #2376)
       # separates multiple extensions that joined by a slash 
-      if QGis.QGIS_VERSION[0:3] < "1.8":
+      if QGis.QGIS_VERSION[0:3] < "1.5":
           formats = self.rastersFilter.split( ";;" )
           self.rastersFilter = QString()
           for f in formats:
