@@ -1,4 +1,35 @@
 # -*- coding: utf-8 -*-
+#-----------------------------------------------------------
+#
+# fTools
+# Copyright (C) 2008-2011  Carson Farmer
+# EMAIL: carson.farmer (at) gmail.com
+# WEB  : http://www.ftools.ca/fTools.html
+#
+# A collection of data management and analysis tools for vector data
+#
+# Geoprocessing functions adapted from 'Geoprocessing Plugin',
+# (C) 2008 by Dr. Horst Duester, Stefan Ziegler
+#-----------------------------------------------------------
+#
+# licensed under the terms of GNU GPL 2
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+#---------------------------------------------------------------------
+
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 from qgis.core import *
@@ -156,13 +187,7 @@ class GeoprocessingDialog( QDialog, Ui_Dialog ):
     self.inShapeA.clear()
     self.inShapeB.clear()
 
-    if self.myFunction == 5 or self.myFunction == 8 or self.myFunction == 3:
-      myListA = ftools_utils.getLayerNames( [ QGis.Point, QGis.Line, QGis.Polygon ] )
-      myListB = ftools_utils.getLayerNames( [ QGis.Polygon ] )
-    elif self.myFunction == 7 or self.myFunction == 6:
-      myListA = ftools_utils.getLayerNames( [ QGis.Point, QGis.Line, QGis.Polygon ] ) # added points and lines to test
-      myListB = ftools_utils.getLayerNames( [ QGis.Point, QGis.Line, QGis.Polygon ] ) # added points and lines to test
-    elif self.myFunction == 4:
+    if self.myFunction == 4:
       myListA = ftools_utils.getLayerNames( [ QGis.Polygon ] )
       myListB = []
     else:
@@ -264,15 +289,15 @@ class geoprocessingThread( QThread ):
       ( self.myParam, useField ) = self.checkParameter( self.vlayerA, self.myParam )
       if not self.myParam is None:
         if self.myFunction == 1:
-          geos, feature, match = self.buffering( useField )
+          geos, feature, match, error = self.buffering( useField )
         elif self.myFunction == 2:
-          geos, feature, match = self.convex_hull( useField )
+          geos, feature, match, error = self.convex_hull( useField )
         elif self.myFunction == 4:
-          geos, feature, match = self.dissolve( useField )
+          geos, feature, match, error = self.dissolve( useField )
     else:
       self.vlayerB = ftools_utils.getVectorLayerByName( self.myLayerB )
       if self.myFunction == 3:
-        geos, feature, match = self.difference()
+        geos, feature, match, error = self.difference()
       elif self.myFunction == 5:
         geos, feature, match, error = self.intersect()
       elif self.myFunction == 6:
@@ -280,7 +305,7 @@ class geoprocessingThread( QThread ):
       elif self.myFunction == 7:
         geos, feature, match, error = self.symetrical_difference()
       elif self.myFunction == 8:
-        geos, feature, match = self.clip()
+        geos, feature, match, error = self.clip()
     self.emit( SIGNAL( "runFinished(PyQt_PyObject)" ), (geos, feature, match, error) )
     self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
 
@@ -296,6 +321,9 @@ class geoprocessingThread( QThread ):
     fields = vproviderA.fields()
     writer = QgsVectorFileWriter( self.myName, self.myEncoding,
     fields, QGis.WKBPolygon, vproviderA.crs() )
+    # check if writer was created properly, if not, return with error
+    if writer.hasError():
+      return GEOS_EXCEPT, FEATURE_EXCEPT, True, writer.errorMessage()
     outFeat = QgsFeature()
     inFeat = QgsFeature()
     inGeom = QgsGeometry()
@@ -422,7 +450,7 @@ class geoprocessingThread( QThread ):
           nElement += 1
           self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), nElement )
     del writer
-    return GEOS_EXCEPT, FEATURE_EXCEPT, True
+    return GEOS_EXCEPT, FEATURE_EXCEPT, True, None
 
   def convex_hull(self, useField ):
     GEOS_EXCEPT = True
@@ -433,6 +461,8 @@ class geoprocessingThread( QThread ):
     fields = vproviderA.fields()
     writer = QgsVectorFileWriter( self.myName, self.myEncoding,
     fields, QGis.WKBPolygon, vproviderA.crs() )
+    if writer.hasError():
+      return GEOS_EXCEPT, FEATURE_EXCEPT, True, writer.errorMessage()
     inFeat = QgsFeature()
     outFeat = QgsFeature()
     inGeom = QgsGeometry()
@@ -554,7 +584,7 @@ class geoprocessingThread( QThread ):
         except:
           GEOS_EXCEPT = False
     del writer
-    return GEOS_EXCEPT, FEATURE_EXCEPT, True
+    return GEOS_EXCEPT, FEATURE_EXCEPT, True, None
 
   def dissolve( self, useField ):
     GEOS_EXCEPT = True
@@ -564,6 +594,8 @@ class geoprocessingThread( QThread ):
     fields = vproviderA.fields()
     writer = QgsVectorFileWriter( self.myName, self.myEncoding,
     fields, vproviderA.geometryType(), vproviderA.crs() )
+    if writer.hasError():
+      return GEOS_EXCEPT, FEATURE_EXCEPT, True, writer.errorMessage()
     inFeat = QgsFeature()
     outFeat = QgsFeature()
     vproviderA.rewind()
@@ -691,7 +723,7 @@ class geoprocessingThread( QThread ):
             outFeat.setAttributeMap( attrs )
             writer.addFeature( outFeat )
     del writer
-    return GEOS_EXCEPT, FEATURE_EXCEPT, True
+    return GEOS_EXCEPT, FEATURE_EXCEPT, True, None
 
   def difference( self ):
     GEOS_EXCEPT = True
@@ -712,6 +744,8 @@ class geoprocessingThread( QThread ):
         crs_match = crsA == crsB
     writer = QgsVectorFileWriter( self.myName, self.myEncoding,
     fields, vproviderA.geometryType(), vproviderA.crs() )
+    if writer.hasError():
+      return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match, writer.errorMessage()
     inFeatA = QgsFeature()
     inFeatB = QgsFeature()
     outFeat = QgsFeature()
@@ -848,7 +882,7 @@ class geoprocessingThread( QThread ):
               FEATURE_EXCEPT = False
               continue
     del writer
-    return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match
+    return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match, None
 
   def intersect( self ):
     GEOS_EXCEPT = True
@@ -1053,80 +1087,110 @@ class geoprocessingThread( QThread ):
     self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0)
     self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
     vproviderA.rewind()
+    count = 0
     while vproviderA.nextFeature( inFeatA ):
       self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), nElement )
       nElement += 1
       found = False
       geom = QgsGeometry( inFeatA.geometry() )
-      diffGeom = QgsGeometry( inFeatA.geometry() )
+      diff_geom = QgsGeometry( geom )
       atMapA = inFeatA.attributeMap()
       intersects = indexA.intersects( geom.boundingBox() )
-      if len( intersects ) <= 0:
+      if len( intersects ) < 1:
         try:
           outFeat.setGeometry( geom )
           outFeat.setAttributeMap( atMapA )
           writer.addFeature( outFeat )
         except:
           FEATURE_EXCEPT = False
-          continue
+          # this really shouldn't happen, as we 
+          # haven't edited the input geom at all
+          # continue
       else:
         for id in intersects:
+          count += 1
           vproviderB.featureAtId( int( id ), inFeatB , True, allAttrsB )
           atMapB = inFeatB.attributeMap()
           tmpGeom = QgsGeometry( inFeatB.geometry() )
           try:
             if geom.intersects( tmpGeom ):
               found = True
-              diff_geom = QgsGeometry( geom.difference( tmpGeom ) )
-              int_geom = QgsGeometry( geom.intersection( tmpGeom ) )
-              if int_geom.wkbType() == 7:
-                int_com = geom.combine( tmpGeom )
-                int_sym = geom.symDifference( tmpGeom )
-                int_geom = QgsGeometry( int_com.difference( int_sym ) )
+              int_geom = geom.intersection( tmpGeom )
+              if int_geom is None:
+                GEOS_EXCEPT = False
+                # There was a problem creating the intersection
+                int_geom = QgsGeometry()
+              else:
+                int_geom = QgsGeometry(int_geom)
+              if diff_geom.intersects( tmpGeom ):
+                diff_geom = diff_geom.difference( tmpGeom )
+                if diff_geom is None:
+                  # It's possible there was an error here?
+                  diff_geom = QgsGeometry()
+                else:
+                  diff_geom = QgsGeometry(diff_geom)
+              if int_geom.wkbType() == 0:
+              # intersection produced different geomety types
+                temp_list = int_geom.asGeometryCollection()
+                for i in temp_list:
+                  if i.type() == geom.type():
+                      int_geom = QgsGeometry( i )
               try:
                 outFeat.setGeometry( int_geom )
                 outFeat.setAttributeMap( ftools_utils.combineVectorAttributes( atMapA, atMapB ) )
+#                print int_geom.wkbType()
                 writer.addFeature( outFeat )
-              except:
+              except Exception, err:
+#                print str(err)
                 FEATURE_EXCEPT = False
-                continue
-            else:
-              try:
-                outFeat.setGeometry( geom )
-                outFeat.setAttributeMap( atMapA )
-                writer.addFeature( outFeat )
-              except:
-                FEATURE_EXCEPT = False
-                continue
-          except:
+#            else:
+#              # this only happends if the bounding box 
+#              # intersects, but the geometry doesn't
+#              try:
+#                outFeat.setGeometry( geom )
+#                outFeat.setAttributeMap( atMapA )
+#                print geom.wkbType()
+#                writer.addFeature( outFeat )
+#              except:
+##                # also shoudn't ever happen
+#                FEATURE_EXCEPT = False
+#              pass
+          except Exception, err:
+#            print str(err)
             GEOS_EXCEPT = False
             found = False
-            continue
         if found:
           try:
+            if diff_geom.wkbType() == 0:
+              temp_list = diff_geom.asGeometryCollection()
+              for i in temp_list:
+                if i.type() == geom.type():
+                    diff_geom = QgsGeometry( i )
             outFeat.setGeometry( diff_geom )
             outFeat.setAttributeMap( atMapA )
+#            print diff_geom.wkbType()
             writer.addFeature( outFeat )
-          except:
+          except Exception, err:
+#            print str(err)
             FEATURE_EXCEPT = False
-            continue
+#            continue
     length = len( vproviderA.fields().values() )
     vproviderB.rewind()
     while vproviderB.nextFeature( inFeatA ):
-      add = True
+      add = False
       geom = QgsGeometry( inFeatA.geometry() )
       diff_geom = QgsGeometry( geom )
       atMap = inFeatA.attributeMap().values()
       atMap = dict( zip( range( length, length + len( atMap ) ), atMap ) )
       intersects = indexB.intersects( geom.boundingBox() )
-      if len(intersects) <= 0:
+      if len(intersects) < 1:
         try:
-          outFeat.setGeometry( diff_geom )
+          outFeat.setGeometry( geom )
           outFeat.setAttributeMap( atMap )
           writer.addFeature( outFeat )
-        except:
+        except Exception, err:
+#          print str(err)
           FEATURE_EXCEPT = False
-          continue
       else:
         for id in intersects:
           vproviderA.featureAtId( int( id ), inFeatB , True, allAttrsA )
@@ -1134,19 +1198,21 @@ class geoprocessingThread( QThread ):
           tmpGeom = QgsGeometry( inFeatB.geometry() )
           try:
             if diff_geom.intersects( tmpGeom ):
+              add = True
               diff_geom = QgsGeometry( diff_geom.difference( tmpGeom ) )
-          except:
+          except Exception, err:
+#            print str(err)
             add = False
             GEOS_EXCEPT = False
-            break
         if add:
           try:
             outFeat.setGeometry( diff_geom )
-            outFeat.setAttributeMap( atMap )
+            outFeat.setAttributeMap( atMapB )
             writer.addFeature( outFeat )
-          except:
+          except Exception, err:
+#            print str(err)
             FEATURE_EXCEPT = False
-            continue
+#            continue
       self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ),  nElement )
       nElement += 1
     del writer
@@ -1264,6 +1330,8 @@ class geoprocessingThread( QThread ):
     fields = vproviderA.fields()
     writer = QgsVectorFileWriter( self.myName, self.myEncoding,
     fields, vproviderA.geometryType(), vproviderA.crs() )
+    if writer.hasError():
+      return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match, writer.errorMessage()
     inFeatA = QgsFeature()
     inFeatB = QgsFeature()
     outFeat = QgsFeature()
@@ -1464,7 +1532,7 @@ class geoprocessingThread( QThread ):
                 GEOS_EXCEPT = False
                 continue
     del writer
-    return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match
+    return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match, None
 
   def checkParameter( self, layer, param ):
     if self.myFunction == 1:
