@@ -1,4 +1,33 @@
 # -*- coding: utf-8 -*-
+#-----------------------------------------------------------
+#
+# fTools
+# Copyright (C) 2008-2011  Carson Farmer
+# EMAIL: carson.farmer (at) gmail.com
+# WEB  : http://www.ftools.ca/fTools.html
+#
+# A collection of data management and analysis tools for vector data
+#
+#-----------------------------------------------------------
+#
+# licensed under the terms of GNU GPL 2
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+#---------------------------------------------------------------------
+
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 import ftools_utils
@@ -65,17 +94,8 @@ class Dialog(QDialog, Ui_Dialog):
             self.outShape.clear()
             addToTOC = QMessageBox.question(self, self.tr("Coordinate statistics"), self.tr("Created output point shapefile:\n%1\n\nWould you like to add the new layer to the TOC?").arg( outPath ), QMessageBox.Yes, QMessageBox.No, QMessageBox.NoButton)
             if addToTOC == QMessageBox.Yes:
-                self.vlayer = QgsVectorLayer(outPath, unicode(outName), "ogr")
-                if self.vlayer.geometryType() == QGis.Point:
-                    render = QgsSingleSymbolRenderer(QGis.Point)
-                    symbol = QgsSymbol(QGis.Point)
-                    symbol.setFillColor(Qt.red)
-                    symbol.setFillStyle(Qt.SolidPattern)
-                    symbol.setColor(Qt.red)
-                    symbol.setPointSize(5)
-                    render.addSymbol(symbol)
-                    self.vlayer.setRenderer(render)
-                QgsMapLayerRegistry.instance().addMapLayer(self.vlayer)
+                vlayer = QgsVectorLayer(outPath, unicode(outName), "ogr")
+                QgsMapLayerRegistry.instance().addMapLayer(vlayer)
         self.progressBar.setValue(0)
         self.buttonOk.setEnabled( True )
 
