@@ -1,34 +1,31 @@
 # -*- coding: utf-8 -*-
 #-----------------------------------------------------------
-# 
-# Generate Random Points
-#
-# A QGIS plugin for generating a simple random points
-# shapefile. 
-#
-# Copyright (C) 2008  Carson Farmer
 #
+# fTools
+# Copyright (C) 2008-2011  Carson Farmer
 # EMAIL: carson.farmer (at) gmail.com
-# WEB  : www.geog.uvic.ca/spar/carson
+# WEB  : http://www.ftools.ca/fTools.html
+#
+# A collection of data management and analysis tools for vector data
 #
 #-----------------------------------------------------------
-# 
+#
 # licensed under the terms of GNU GPL 2
-# 
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License along
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-# 
+#
 #---------------------------------------------------------------------
 
 from PyQt4.QtCore import *
@@ -116,10 +113,7 @@ class Dialog(QDialog, Ui_Dialog):
                 value = self.spnUnstratified.value()
             else:
                 QMessageBox.information(self, self.tr("Random Points"), self.tr("Unknown layer type..."))
-            if self.chkMinimum.isChecked():
-                minimum = self.spnMinimum.value()
-            else:
-                minimum = 0.00
+            minimum = 0.00
             self.progressBar.setValue(10)
             self.randomize(inLayer, outPath, minimum, design, value)
             self.progressBar.setValue(100)
