@@ -79,6 +79,7 @@ authorRepos  = [("Aaron Racicot's Repository", "http://qgisplugins.z-pulley.com"
                 ("CatAIS Repository",          "http://www.catais.org/qgis/plugins.xml", ""),
                 ("Faunalia Repository",        "http://www.faunalia.it/qgis/plugins.xml", "http://faunalia.it/qgis/plugins.xml"),
                 ("GIS-Lab Repository",         "http://gis-lab.info/programs/qgis/qgis-repo.xml", ""),
+                ("Kappasys Repository",        "http://www.kappasys.org/qgis/plugins.xml", ""),
                 ("Martin Dobias' Sandbox",     "http://mapserver.sk/~wonder/qgis/plugins-sandbox.xml", ""),
                 ("Marco Hugentobler's Repository","http://karlinapp.ethz.ch/python_plugins/python_plugins.xml", ""),
                 ("Sourcepole Repository",      "http://build.sourcepole.ch/qgis/plugins.xml", ""),
@@ -435,11 +436,8 @@ class Repositories(QObject):
             if QGIS_VER[0]==qgisMinimumVersion[0] or (qgisMinimumVersion!="0" and qgisMaximumVersion!="2"):   # to be deleted
               #add the plugin to the cache
               plugins.addFromRepository(plugin)
-        self.mRepositories[reposName]["state"] = 2
-      else:
-        #print "Repository parsing error"
-        self.mRepositories[reposName]["state"] = 3
-        self.mRepositories[reposName]["error"] = QCoreApplication.translate("QgsPluginInstaller","Couldn't parse output from the repository")
+      # set state=2, even if the repo is empty
+      self.mRepositories[reposName]["state"] = 2
 
     self.emit(SIGNAL("repositoryFetched(QString)"), reposName )
 
